#include "assert.h"
#include "contact.h"

extern gsl_rng* random_number_generator;

bool contact(
		const double r_transmission,
		const double r_removal,
		const double r_sigma,
		const double r_width,
		const int num_particles_max,
		const int trials
		) {

	// In the distributed-contact limit,
	// we only have to keep track of the 
	// anchor-location of each individual
	const int anchor	= 0;
	const double p_removal = r_removal/(r_removal+r_transmission);
	const double r_kernel_width = sqrt( r_sigma*r_sigma + r_width*r_width);

	for ( int tc = 1; tc <= trials; ++tc ) {
		// [ [ time, position, anchor ] * n ]
		double* __restrict__ population;
		population = new double[num_particles_max]; 
		// Initial particle
		double t = 0.0;
		population[1*0+anchor] = 0.0;
		double end_pos = population[1*0+anchor];
		double end_neg = population[1*0+anchor];
		// First edges
		int num_particles = 1;
		printf("%f\t%f\t%d\t%d\n",t,end_pos,num_particles,tc);
		while (0 < num_particles && num_particles < num_particles_max) {
			const int p = gsl_rng_uniform_int(random_number_generator,num_particles);
            assert( p > -1 and p < num_particles );
            const int dp_removal = gsl_ran_bernoulli(random_number_generator, p_removal);
            double dt = gsl_ran_exponential(random_number_generator,
				1./( (r_transmission+r_removal) * double(num_particles) ) );
            t += dt;
            if (1==dp_removal) {
				--num_particles;
				if ( 0 == num_particles ) continue;
				if ( p == num_particles ) continue; 
				population[1*p+anchor] = population[1*num_particles+anchor];
			} else { 
				double p_a;

				// For comparison
				//p_a = population[1*p+anchor]
				//	+ gsl_ran_ugaussian(random_number_generator)*r_sigma
				//	+ gsl_ran_ugaussian(random_number_generator)*r_width;
				p_a = population[1*p+anchor]
					+ gsl_ran_ugaussian(random_number_generator)*r_kernel_width;
				population[1*num_particles+anchor] = p_a;
				++num_particles;
				
				if ( end_pos < p_a ) {
					end_pos = p_a ; // max operator
					printf("%f\t%f\t%d\t%d\n",t,end_pos,num_particles,tc);
				}

			} // end fission if statement
		} // end while loop
		printf("\n");
		delete population;
	}
	return 0;
}
