#include "diffusion.h"
#include "assert.h"

extern gsl_rng* random_number_generator;

bool diffusion(
		const double r_transmission,
		const double r_removal,
		const double r_diffusion,
		const int num_particles_max,
		const int trials
		) {


	// indexing variables, used to speed up code
	const int time = 0;
	const int position = 1;
	const int N = ( time > position ? time : position ) + 1;

	const double r_sigma = sqrt(2*r_diffusion);
	const double p_removal = r_removal/(r_removal+r_transmission);
	double* __restrict__ population;
	for ( int tc = 1; tc <= trials; ++tc ) {
		// [ [ time, position ] * n ]
		population = new double[N*num_particles_max]; 
		// Initial particle
		double t = 0.0;
		population[time] = 0.0;
		population[position] = 0.0;
		double end_pos = population[N*0+position];
		// First edges
		int num_particles = 1;
		printf("%f\t%f\t%d\t%d\n",t,end_pos,num_particles,tc);
		while (0 < num_particles && num_particles < num_particles_max) {
			const int p = gsl_rng_uniform_int(random_number_generator,num_particles);
			assert( p > -1 and p < num_particles );
			const int dp_removal = gsl_ran_bernoulli(random_number_generator, p_removal);
			double dt = gsl_ran_exponential(random_number_generator, 1.)
                / ( (r_transmission+r_removal) * double(num_particles) ) ;
            t += dt;
			if (1==dp_removal) {
				--num_particles;
				if (num_particles == 0 ) continue;
				if (num_particles == p ) continue;
				population[N*p+time] = population[N*num_particles+time];
				population[N*p+position] = population[N*num_particles+position];
			} else { 
				// If the next event is a tranmission
				double p_t, p_std, p_x;
				p_t = t - population[N*p+time];
				p_x = population[N*p+position]
					+ gsl_ran_gaussian(random_number_generator, r_sigma*sqrt(p_t));
				population[N*p+time] = t;
				population[N*p+position] = p_x;
				population[N*num_particles+time] = t;
				population[N*num_particles+position] = p_x;

				++num_particles;

				// For DEBUGGING only.  Synchs with uhlenbeck version
				//gsl_ran_ugaussian(random_number_generator);

				if ( p_x > end_pos ) {
					end_pos = p_x ; 
					printf("%f\t%f\t%d\t%d\n",t,end_pos,num_particles,tc);
				}
			} // end fission if statement
		} // end while loop
		printf("\n");
		delete population;
	}
	return 0;
}
