#import "hyperbolic.h"

extern gsl_rng* random_number_generator;

bool hyperbolic(
		const double r_transmission,
		const double r_removal,
		const double r_width,
		const double r_alpha,
		const int num_particles_max,
		const int trials
		) {
	const int time = 1;
	const int position = 0;
	const int speed = 2;
	const int N = 3;

	const double sigma_c = r_alpha*r_width;

	const double p_removal = r_removal/(r_removal + r_transmission);
	double* __restrict__ population; 
	for ( int tc = 1; tc <= trials; ++tc) {
		population = new double[num_particles_max*N];
		int num_particles = 1;
		population[N*num_particles+time] = 0.;
		population[N*num_particles+position] = 0.;
		population[N*num_particles+speed] = 0.;
		double end_pos = 0.0;
		double t = 0.0;
		printf("%f\t%f\t%d\t%d\n",t,end_pos,num_particles,tc);
		while ( 0 < num_particles and  num_particles < num_particles_max ) {
			const int p = gsl_rng_uniform_int(random_number_generator,num_particles);
			assert( p > -1 and p < num_particles );
			const int dp_removal = gsl_ran_bernoulli(random_number_generator, p_removal);
			double dt = gsl_ran_exponential(random_number_generator,
					1./( (r_transmission+r_removal) * double(num_particles) ) );
			t += dt;
			if ( 0 == dp_removal ) {
				double dnt = t - population[N*p+time];
				double nx = population[N*p+position];
				double nc = population[N*p+speed];
				nx += nc * dnt;
				population[N*p+time] = t;
				population[N*p+position] = nx;
				population[N*num_particles+time] = t;
				population[N*num_particles+position] = nx;
				population[N*num_particles+speed] =
					sigma_c*gsl_ran_ugaussian(random_number_generator);
				++num_particles;
				if ( nx > end_pos ) {
					end_pos = nx;
					printf("%f\t%f\t%d\t%d\n",t,end_pos,num_particles,tc);
				}
			} else if ( 1 == dp_removal ) {
				//printf("dying\n");
				--num_particles;
				if ( p == num_particles ) continue; 
				if ( 0 == num_particles ) continue; 
				population[N*p+time] = population[N*num_particles+time];
				population[N*p+speed] = population[N*num_particles+speed];
				population[N*p+position] = population[N*num_particles+position];
			} else {
				assert(false); 
			}
		}
		printf("\n");
		delete population;
	}
	// Last edges
	return 0;
}
