
#import "mymath.h"

#import <cmath>
#import <cstdlib>
#import <cstdio>
#import <assert.h>
//#include <ctype.h>
//#include <iostream.h>
//#include <iomanip>

//#include "myassert.h"

//Quick sorting: p and q serve as indexes into the array
//being sorted.  T is the class.  A is an array of pointers
//to the class, which has an ordering imposed.
template<class T>
void quicksort(T* A[], long p, long r) {
        long q;
        if (p<r) {
                q = partition(A,p,r);
                quicksort(A,p,q);
                quicksort(A,q+1,r);
        }
        return;
}

// This is a function internal to quicksort, and
// never should be called by itself.
template<class T>
inline long partition(T* A[], long p, long r) {
        T* x1 = A[p];
        T* x2;
        p--; r++;
        while (true) {
                for (p++; *A[p] > *x1; p++);
                for (r--; *A[r] < *x1; r--);
                if (p<r) {
                        x2 = A[p];
                        A[p] = A[r];
                        A[r] = x2;
                } else return r;
        }
}

template<class T>
inline T* max(T* a, T* b) {
	if (*a > *b) return a;
	return b;
}

template<class T>
inline T* min(T* a, T* b) {
	if (*a > *b) return b;
	return a;
}

double magnitude(double x, double y) {
	return sqrt( x * x + y * y);
}

long sign(long x) {
	if (x<0) return -1;
	else if (x>0) return 1;
	else return 0;
}

double heavyside(double x) {
	if (x <0.0) return 0.0;
	else if (x > 0.0) return 1.0;
	else return 0.5;
}

int factorial(int x) {
	int fval;
	fval = 1;
	while (x >0) {
		fval = fval * x;
		x--;
	}
	return(fval);
}

int gcd(int p, int q) {
	int x;
	if (p < q) { x = p; p = q; q = x ;}
	for (; x = p % q; ) {
		p = q;
		q = x;
	}
	return q;
}

double nCr(double n, double r) {
        double rval=1;
        int x = int(n);
        int y = int(r);
        for (int i = x; i>x-y; i--) rval*=double(i);
        for (int i = y; i>1; i--) rval = rval/double(i);
        return(rval);
}

double binomial(double n, double r, double p) {
        double rval = 1;
        int N = int(n);
        int R = int(r);
        rval = nCr(n,r) * pow(p,R) * pow(1-p,N-R);
        return rval;
}

double poisson(double x, double lambda) {
	double rval=0;
	double x_factorial = double(factorial(int(x)));
	rval = exp(-lambda)* pow(lambda,x)/x_factorial;
	return rval;
}

double uniform_random(void) {
        return double(random())/double(RAND_MAX);
}
inline int int_random(int max) {
	int x = max;
	while ( x >= max) {
		x = int( double(max)*uniform_random() );
	}
	return x;
}
bool bool_random(double p) {
	return (p > uniform_random()) ? true : false;
}
unsigned int getrandomseed(void) {
	unsigned int seed;
	FILE *fp;

	if ( (fp = fopen("/dev/random", "r")) == NULL ) {
		fprintf(stderr,"Can't open /dev/random for reading.\n");
		exit(0);
	}
	fread(&seed, sizeof(unsigned int), 1, fp);
	fclose(fp);
	return seed;
}

void gaussian_elimination(double* A, double* b, const int n) {
	/* Gaussian elimination with partial pivoting */
	/* The implementation is streamlined, but not the most
	 * efficient in several places */
	// assume A is n*n, stored in column form
	//for (int ii =0; ii< n*n; ++ii) printf("%f\t",A[ii]); printf("\n");
	//for (int ii =0; ii< n; ++ii) printf("%f\t",b[ii]); printf("\n");
	int *P = (int*)(calloc(n,sizeof(int)));
	//int *U = calloc(n,sizeof(int));
	for (int i =0; i< n; ++i) { P[i] = i;}
	for (int i =0; i< n; ++i) {
		{
			double pv = A[P[i]+n*i];
			for (int j=i+1; j<n; ++j) {
				assert( j < n) ;
				if (fabs(A[P[j]+n*i]) > pv) {
					pv = fabs(A[P[j]+n*i]);
					int tmp = P[i];
					P[i]=P[j];
					P[j]=tmp;
				}
			}
		}
		
		int e = P[i];
		double a = 1./A[e+n*i];
		// scale equation
		b[e] = b[e]*a;
		for ( int j=i; j<n; ++j) { A[e+j*n]*=a; }
		// eliminate for all other equations
		for ( int k=i+1; k<n; ++k) {
			assert(k<n);
			int f = P[k];
			double c = A[f+i*n];
			b[f] -= c*b[e];
			for ( int j=i; j<n; ++j) { A[f+j*n]-= c*A[e+j*n]; }
		}
		//for (int ii =0; ii< n*n; ++ii) printf("%f\t",A[ii]); printf("\n");
		//for (int ii =0; ii< n; ++ii) printf("%f\t",b[ii]); printf("\n");
	}
	for (int i =n-1; i>=0 ; --i) {
		int e = P[i];
		for ( int k=i-1; k>=0; --k) {
			int f = P[k];
			b[f] -= A[f+i*n]*b[e];
			A[f+i*n] = 0;
		}
		//for (int ii =0; ii< n*n; ++ii) printf("%f\t",A[ii]); printf("\n");
		//for (int ii =0; ii< n; ++ii) printf("%f\t",b[ii]); printf("\n");
	}
	double* x = (double *)(calloc(n,sizeof(double)));
	for (int i =0; i< n; ++i) { x[i] = b[P[i]]; }
	for (int i =0; i< n; ++i) { b[i] = x[i]; }
	free(x);
	//for (int ii =0; ii< n; ++ii) printf("%d\t",P[ii]); printf("\n");
}

/*
// Elimination, but without pivoting.
void gaussian_elimination(double* A, double* b, const int n) {
	// assume A is n*n, stored in column form
	for (int i =0; i< n; ++i) {
		double a = 1./A[i+n*i];
		b[i] = b[i]*a;
		for ( int j=i; j<n; ++j) { A[i+j*n]*=a; }
		for ( int k=i+1; k<n; ++k) {
			assert(k<n);
			double c = A[k+i*n];
			b[k] -= c*b[i];
			for ( int j=i; j<n; ++j) { A[k+j*n]-= c*A[i+j*n]; }
		}
		//for (int ii =0; ii< n*n; ++ii) printf("%f\t",A[ii]);
		//printf("\n");
	}
	for (int i =n-1; i>=0 ; --i) {
		for ( int k=i-1; k>=0; --k) {
			double c = A[k+i*n];
			A[k+i*n] -= c;
			b[k] -= c*b[i];
		}
	}
} */

