#import "uhlenbeck.h"

extern gsl_rng* random_number_generator;

bool uhlenbeck(
		const double r_transmission,
		const double r_removal,
		const double r_sigma,
		const double r_width,
		const double r_alpha,
		const int num_particles_max,
		const int trials,
		const bool measureAnchors
		) {

	const int time		= 2;
	const int position	= 1;
	const int anchor	= 0;

	const double ratio_removed = r_removal/(r_removal+r_transmission);

	for ( int tc = 1; tc <= trials; ++tc ) {
		// [ [ time, position, anchor ] * n ]
		double* __restrict__ population;
		population = new double[3*num_particles_max]; 
		// Initial particle
		double t = 0.0;
		population[3*0+time] = t;
		population[3*0+position] = 0.0;
		population[3*0+anchor] = 0.0;
		double end_pos = population[3*0+position];
		double end_neg = population[3*0+position];
		// First edges
		printf("%f\t%f\t%d\t%d\n",t,end_pos,1,tc);
		int num_particles = 1;
		while (0 < num_particles && num_particles < num_particles_max) {
			const int p = gsl_rng_uniform_int(random_number_generator,num_particles);
				// return an integer from 0 to num_particles - 1,
				// but not num_particles
			if (1==gsl_ran_bernoulli(random_number_generator,ratio_removed)) {
				// If the next event is a removal
				const double dt = gsl_ran_exponential(random_number_generator,
					1./(r_removal*num_particles));
				t+= dt;
				--num_particles;
				// if that was the last, go on
				// if the dying particle is at the end, go on
				if (num_particles != 0 && num_particles != p ) {
					// otherwise, move the last particle to position of
					// the dying particle
					// This is pretty clever, dude.  No wasted memory.
					population[3*p+time] = population[3*num_particles+time];
					population[3*p+position] = population[3*num_particles+position];
					population[3*p+anchor] = population[3*num_particles+anchor];
				}
			} else { 
				// If the next event is a tranmission
				const double dt = gsl_ran_exponential(random_number_generator,
					1./(r_transmission*num_particles));

				double p_t, decay, p_std, p_a, p_x;

				t+= dt;
				// calculation the particle position
				p_t = t - population[3*p+time];
				decay = exp(-r_alpha*p_t);
				p_std = r_sigma * sqrt( 1. - decay*decay);
				p_a = population[3*p+anchor];
				p_x = p_a
					+ (population[3*p+position]-p_a)*decay
					+ gsl_ran_gaussian(random_number_generator,p_std);
				// update the particle
				population[3*p+time] = t;
				population[3*p+position] = p_x;
				// calculate the anchor of new particle p
				p_a = p_x + gsl_ran_gaussian(random_number_generator,r_width);
				// create particle 
				population[3*num_particles+time] = t;
				population[3*num_particles+position] = p_x;
				population[3*num_particles+anchor] = p_a;
				++num_particles;

				// Do output only when a new extreme individual can appear
				if ( measureAnchors ) {
					// The offspring particle may have an anchor more extreme
					//end_neg = end_neg <? p_a ; // min operator
					if ( end_pos < p_a ) {
						end_pos = p_a ; // max operator
						printf("%f\t%f\t%d\t%d\n",t,end_pos,num_particles,tc);
					}
				} else {
					// The parent and offspring have the same positions
					if ( end_pos < p_x ) {
						end_pos = p_x ; 
						printf("%f\t%f\t%d\t%d\n",t,end_pos,num_particles,tc);
					}
				}
			} // end fission if statement
		} // end while loop
		printf("\n");
		delete population;
	}
	return 0;
}
